#version 330 core

in vec2 texcoords;

uniform sampler2D ligthSourceTex;
uniform sampler2D renderTex;
uniform sampler2D depthTex;

uniform vec3 lightPosScreenSpace;
uniform vec3 chromaticDistortion;
uniform float halowidth;
uniform float ghostBrightness;
uniform float haloBrightness;

layout(location = 0) out vec4 fragColor;

const int numSamples = 5;

vec4 textureDistorted(in sampler2D tex, in vec2 texcoord, in vec2 direction, in vec3 distortion ) 
{
return vec4(texture(tex, texcoord + direction * distortion.r).r,
	    texture(tex, texcoord + direction * distortion.g).g,
	    texture(tex, texcoord + direction * distortion.b).b,
	    1.0);
}

void main(void)
{
//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-is ligth source visible-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
//in chase of one Pixel objects, check neighbourpixels too
    float depth1 = texture(depthTex, lightPosScreenSpace.xy).w; 
    float depth2 = texture(depthTex, lightPosScreenSpace.xy+vec2(0.01, 0.01)).w;
    float depth3 = texture(depthTex, lightPosScreenSpace.xy-vec2(0.01, 0.01)).w;
    float depth4 = texture(depthTex, lightPosScreenSpace.xy+vec2(-0.01, 0.01)).w;
    float depth5 = texture(depthTex, lightPosScreenSpace.xy-vec2(-0.01, 0.01)).w;     

    float mindepth = depth1;

    mindepth = min(mindepth, depth2);
    mindepth = min(mindepth, depth3);
    mindepth = min(mindepth, depth4);
    mindepth = min(mindepth, depth5);

    if(mindepth < 0.0)
        mindepth = 1000000.0;
    

    float maxdepth = depth1;

    maxdepth = max(maxdepth, depth2);
    maxdepth = max(maxdepth, depth3);
    maxdepth = max(maxdepth, depth4);
    maxdepth = max(maxdepth, depth5);

    if(maxdepth < 0.0)
        maxdepth = 1000000.0;
//-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-.-
    if(mindepth < lightPosScreenSpace.z-0.125 && maxdepth < lightPosScreenSpace.z-0.125)
    {
         fragColor = texture(renderTex, texcoords); //if the ligth source is not visible on this position, draw the normal fragmentcolor
    }
    else	//else, draw the flairs
    {
	vec4 result = vec4(0.0);

        
	vec2 texCoordFlip = -texcoords + vec2(1.0); 
	vec2 coords;

	vec2 deltaTexCoord = 0.75*(texcoords-texCoordFlip)*0.25;


	vec2 FlareDir = (texcoords-texCoordFlip);
	FlareDir = normalize(FlareDir);
	vec2 HaloDir = FlareDir*halowidth;

	
	for(int i = 0; i < numSamples; i++)
	{
	coords = fract(texCoordFlip+deltaTexCoord*float(i));
        result += ghostBrightness*textureDistorted(ligthSourceTex, coords, FlareDir, chromaticDistortion);
	}

        // Halo-Darstellung: 
	
        result += textureDistorted(ligthSourceTex, fract(texCoordFlip + HaloDir), FlareDir, chromaticDistortion)  *haloBrightness;
        // Hinweis: fract gibt den Nachkommateil zurck

        fragColor = result+texture(renderTex, texcoords);
    }
}