#version 330 core

struct Light {
    vec3 position;
    vec3 color;
    float linear;
    float quadratic;
    float radius;
};

layout (location = 0) in vec3 position;
layout (location = 1) in vec3 normal;
layout (location = 2) in vec2 texCoord;

layout (location = 3) in vec3 tangent;
layout (location = 4) in vec3 bitangent;

uniform mat4 model;
uniform mat4 modelview;
uniform mat4 modelViewProjectionMatrix;
uniform mat3 normalMatrix;
uniform mat4 lightMPV;
uniform vec4 u_plane0 = vec4(0.0,1.0,0.0,0.0); // in viewspace, fuers clippen

const vec4 u_plane0_2 = vec4(0.0,1.0,0.0,0.0);

uniform vec3 lightPos;
uniform vec3 eyePos;
uniform Light pointLight1;
uniform Light pointLight2;

uniform bool hasBumpMapTexture = true;

out vec2 _texUV;
smooth out vec3 _normal;

out vec3 _eyePos;
out vec4 _fragPosLightSpace;
out vec3 _fragPos;
out vec3 _pointlightPos1;
out vec3 _pointlightPos2;
out vec3 _lightPos;

out vec3 tmpT;
out vec3 tmpBT;

void main() 
{
    _texUV = texCoord;

    gl_Position = modelViewProjectionMatrix * vec4(position, 1);
    _normal =  normalize(normalMatrix * normal);

    vec4 posviewspace = model * vec4(position, 1);

if(hasBumpMapTexture)
{  
   vec3 T = normalize(vec3(model * vec4(tangent,   0.0)));
   vec3 B = normalize(vec3(model * vec4(bitangent, 0.0)));
   vec3 N = normalize(vec3(model * vec4(normal,    0.0)));
   mat3 TBN = transpose(mat3(T, B, N));


   _eyePos = TBN * eyePos;
   _pointlightPos1  = TBN * pointLight1.position;
   _pointlightPos2  = TBN * pointLight2.position;
   _fragPos  = TBN * vec3(model * vec4(position, 0.0));
   _lightPos = TBN * lightPos;
}else{
   _eyePos = eyePos;
   _pointlightPos1  = pointLight1.position;
   _pointlightPos2  = pointLight2.position;
   _fragPos  = vec3(model * vec4(position, 0.0));
   _lightPos = lightPos;

}
	//Erste Zeile ist die const plane. Zweite Zeile die uniform plane.
	//gl_ClipDistance[0] = dot(u_plane0_2,posviewspace);
	gl_ClipDistance[0] = dot(u_plane0,posviewspace);
	
    _fragPosLightSpace = lightMPV * posviewspace; 
}
